@@ -16,16 +16,16 @@
 import re
 
 import netaddr
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
 from neutron.common import ipv6_utils
-from neutron.db import api as db_api
 from neutron.db import common_db_mixin as base_db
-from neutron import manager
-from neutron.plugins.common import constants
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins import constants as pg_const
+from neutron_lib.plugins import directory
 from oslo_db import exception
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -33,11 +33,13 @@ from oslo_utils import uuidutils
 from sqlalchemy import orm
 from sqlalchemy.orm import exc
 from sqlalchemy.orm import lazyload
+from sqlalchemy.orm import subqueryload
 
 from neutron_lbaas._i18n import _
 from neutron_lbaas import agent_scheduler
 from neutron_lbaas.db.loadbalancer import models
 from neutron_lbaas.extensions import l7
+from neutron_lbaas.extensions import lb_network_vip
 from neutron_lbaas.extensions import loadbalancerv2
 from neutron_lbaas.extensions import sharedpools
 from neutron_lbaas.services.loadbalancer import constants as lb_const
@@ -57,7 +59,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
 
     @property
     def _core_plugin(self):
-        return manager.NeutronManager.get_plugin()
+        return directory.get_plugin()
 
     def _get_resource(self, context, model, id, for_update=False):
         resource = None
@@ -92,9 +94,11 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             return False
         return True
 
-    def _get_resources(self, context, model, filters=None):
+    def _get_resources(self, context, model, filters=None, options=None):
         query = self._get_collection_query(context, model,
                                            filters=filters)
+        if options:
+            query = query.options(options)
         return [model_instance for model_instance in query]
 
     def _create_port_choose_fixed_ip(self, fixed_ips):
@@ -196,10 +200,21 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         except n_exc.SubnetNotFound:
             raise loadbalancerv2.EntityNotFound(name="Subnet", id=subnet_id)
 
+    def _validate_and_return_vip_net(self, ctxt, lb):
+        network_id = lb.pop('vip_network_id', None)
+
+        if network_id != n_const.ATTR_NOT_SPECIFIED and network_id:
+            subnets = self._core_plugin.get_subnets_by_network(ctxt,
+                                                               network_id)
+            if not subnets:
+                raise lb_network_vip.VipNetworkInvalid(network=network_id)
+            return network_id
+        return
+
     def assert_modification_allowed(self, obj):
         status = getattr(obj, 'provisioning_status', None)
-        if status in [constants.PENDING_DELETE, constants.PENDING_UPDATE,
-                      constants.PENDING_CREATE]:
+        if status in [n_const.PENDING_DELETE, n_const.PENDING_UPDATE,
+                      n_const.PENDING_CREATE]:
             id = getattr(obj, 'id', None)
             raise loadbalancerv2.StateInvalid(id=id, state=status)
 
@@ -222,13 +237,13 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             # Otherwise we are just setting the load balancer's provisioning
             # status to the status passed in
             if db_lb_child:
-                db_lb.provisioning_status = constants.PENDING_UPDATE
+                db_lb.provisioning_status = n_const.PENDING_UPDATE
                 db_lb_child.provisioning_status = status
             else:
                 db_lb.provisioning_status = status
 
     def update_loadbalancer_provisioning_status(self, context, lb_id,
-                                                status=constants.ACTIVE):
+                                                status=n_const.ACTIVE):
         self.update_status(context, models.LoadBalancer, lb_id,
                            provisioning_status=status)
 
@@ -249,9 +264,13 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             if provisioning_status and (model_db.provisioning_status !=
                                         provisioning_status):
                 model_db.provisioning_status = provisioning_status
+                LOG.debug("Provisioning status of %s (id=%s) updated to: %s",
+                          model_db.NAME, model_db.id, provisioning_status)
             if (operating_status and hasattr(model_db, 'operating_status') and
                     model_db.operating_status != operating_status):
                 model_db.operating_status = operating_status
+                LOG.debug("Operating status of %s (id=%s) updated to: %s",
+                          model_db.NAME, model_db.id, operating_status)
 
     def create_loadbalancer_graph(self, context, loadbalancer,
                                   allocate_vip=True):
@@ -315,12 +334,13 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
 
     def create_loadbalancer(self, context, loadbalancer, allocate_vip=True):
         self._load_id(context, loadbalancer)
-        vip_network_id = loadbalancer.pop('vip_network_id', None)
+        vip_network_id = self._validate_and_return_vip_net(context,
+                                                           loadbalancer)
         vip_subnet_id = loadbalancer.pop('vip_subnet_id', None)
         vip_address = loadbalancer.pop('vip_address')
         if vip_subnet_id and vip_subnet_id != n_const.ATTR_NOT_SPECIFIED:
             loadbalancer['vip_subnet_id'] = vip_subnet_id
-        loadbalancer['provisioning_status'] = constants.PENDING_CREATE
+        loadbalancer['provisioning_status'] = n_const.PENDING_CREATE
         loadbalancer['operating_status'] = lb_const.OFFLINE
         lb_db = models.LoadBalancer(**loadbalancer)
 
@@ -364,7 +384,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             return
         if port_db['device_owner'] == n_const.DEVICE_OWNER_LOADBALANCERV2:
             filters = {'vip_port_id': [port_id]}
-            if len(self.get_loadbalancers(context, filters=filters)) > 0:
+            if len(self.get_loadbalancer_ids(context, filters=filters)) > 0:
                 reason = _('has device owner %s') % port_db['device_owner']
                 raise n_exc.ServicePortInUse(port_id=port_db['id'],
                                              reason=reason)
@@ -374,12 +394,33 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             _prevent_lbaasv2_port_delete_callback, resources.PORT,
             events.BEFORE_DELETE)
 
+    def get_loadbalancer_ids(self, context, filters=None):
+        lb_dbs = self._get_resources(context, models.LoadBalancer,
+                                     filters=filters)
+        return [lb_db.id
+                for lb_db in lb_dbs]
+
     def get_loadbalancers(self, context, filters=None):
         lb_dbs = self._get_resources(context, models.LoadBalancer,
                                      filters=filters)
         return [data_models.LoadBalancer.from_sqlalchemy_model(lb_db)
                 for lb_db in lb_dbs]
 
+    def get_loadbalancers_as_api_dict(self, context, filters=None):
+        options = (
+            subqueryload(models.LoadBalancer.listeners),
+            subqueryload(models.LoadBalancer.pools),
+            subqueryload(models.LoadBalancer.provider)
+        )
+        lb_dbs = self._get_resources(context, models.LoadBalancer,
+                                     filters=filters, options=options)
+        return [lb_db.to_api_dict
+                for lb_db in lb_dbs]
+
+    def get_provider_names_used_in_loadbalancers(self, context):
+        lb_dbs = self._get_resources(context, models.LoadBalancer)
+        return [lb_db.provider.provider_name for lb_db in lb_dbs]
+
     def get_loadbalancer(self, context, id):
         lb_db = self._get_resource(context, models.LoadBalancer, id)
         return data_models.LoadBalancer.from_sqlalchemy_model(lb_db)
@@ -427,8 +468,8 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             if pool.loadbalancer_id != listener.loadbalancer_id:
                 raise sharedpools.ListenerAndPoolMustBeOnSameLoadbalancer()
 
-        if (l7policy['action'] == lb_const.L7_POLICY_ACTION_REDIRECT_TO_URL
-            and 'redirect_url' not in l7policy):
+        if (l7policy['action'] == lb_const.L7_POLICY_ACTION_REDIRECT_TO_URL and
+            'redirect_url' not in l7policy):
             raise l7.L7PolicyRedirectUrlMissing()
 
     def _validate_l7rule_data(self, context, rule):
@@ -494,7 +535,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         try:
             with context.session.begin(subtransactions=True):
                 self._load_id(context, listener)
-                listener['provisioning_status'] = constants.PENDING_CREATE
+                listener['provisioning_status'] = n_const.PENDING_CREATE
                 listener['operating_status'] = lb_const.OFFLINE
                 # Check for unspecified loadbalancer_id and listener_id and
                 # set to None
@@ -557,6 +598,17 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.Listener.from_sqlalchemy_model(listener_db)
                 for listener_db in listener_dbs]
 
+    def get_listeners_as_api_dict(self, context, filters=None):
+        options = (
+            subqueryload(models.Listener.sni_containers),
+            subqueryload(models.Listener.loadbalancer),
+            subqueryload(models.Listener.l7_policies)
+        )
+        listener_dbs = self._get_resources(context, models.Listener,
+                                           filters=filters, options=options)
+        return [listener_db.to_api_dict
+                for listener_db in listener_dbs]
+
     def get_listener(self, context, id):
         listener_db = self._get_resource(context, models.Listener, id)
         return data_models.Listener.from_sqlalchemy_model(listener_db)
@@ -600,7 +652,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
     def create_pool(self, context, pool):
         with context.session.begin(subtransactions=True):
             self._load_id(context, pool)
-            pool['provisioning_status'] = constants.PENDING_CREATE
+            pool['provisioning_status'] = n_const.PENDING_CREATE
             pool['operating_status'] = lb_const.OFFLINE
 
             session_info = pool.pop('session_persistence', None)
@@ -613,7 +665,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
 
             context.session.add(pool_db)
         context.session.refresh(pool_db.loadbalancer)
-        return data_models.Pool.from_sqlalchemy_model(pool_db)
+        return self.get_pool(context, pool_db.id)
 
     def update_pool(self, context, id, pool):
         with context.session.begin(subtransactions=True):
@@ -660,8 +712,9 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
                                      {'default_pool_id': None})
             for l in pool_db.loadbalancer.listeners:
                 for p in l.l7_policies:
-                    if (p.action == lb_const.L7_POLICY_ACTION_REDIRECT_TO_POOL
-                        and p.redirect_pool_id == id):
+                    if (p.action ==
+                        lb_const.L7_POLICY_ACTION_REDIRECT_TO_POOL and
+                        p.redirect_pool_id == id):
                         self.update_l7policy(
                             context, p.id,
                             {'redirect_pool_id': None,
@@ -673,6 +726,19 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.Pool.from_sqlalchemy_model(pool_db)
                 for pool_db in pool_dbs]
 
+    def get_pools_as_api_dict(self, context, filters=None):
+        options = (
+            subqueryload(models.PoolV2.members),
+            subqueryload(models.PoolV2.listeners),
+            subqueryload(models.PoolV2.l7_policies),
+            subqueryload(models.PoolV2.loadbalancer),
+            subqueryload(models.PoolV2.session_persistence)
+        )
+        pool_dbs = self._get_resources(context, models.PoolV2,
+            filters=filters, options=options)
+        return [pool_db.to_api_dict
+                for pool_db in pool_dbs]
+
     def get_pool(self, context, id):
         pool_db = self._get_resource(context, models.PoolV2, id)
         return data_models.Pool.from_sqlalchemy_model(pool_db)
@@ -682,7 +748,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             with context.session.begin(subtransactions=True):
                 self._load_id(context, member)
                 member['pool_id'] = pool_id
-                member['provisioning_status'] = constants.PENDING_CREATE
+                member['provisioning_status'] = n_const.PENDING_CREATE
                 member['operating_status'] = lb_const.OFFLINE
                 member_db = models.MemberV2(**member)
                 context.session.add(member_db)
@@ -712,6 +778,13 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.Member.from_sqlalchemy_model(member_db)
                 for member_db in member_dbs]
 
+    def get_pool_members_as_api_dict(self, context, filters=None):
+        filters = filters or {}
+        member_dbs = self._get_resources(context, models.MemberV2,
+                                         filters=filters)
+        return [member_db.to_api_dict
+                for member_db in member_dbs]
+
     def get_pool_member(self, context, id):
         member_db = self._get_resource(context, models.MemberV2, id)
         return data_models.Member.from_sqlalchemy_model(member_db)
@@ -725,11 +798,12 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         with context.session.begin(subtransactions=True):
             hm_db = self.create_healthmonitor(context, healthmonitor)
             pool = self.get_pool(context, pool_id)
-            # do not want listener, members, healthmonitor or loadbalancer
-            # in dict
+            # do not want listener, members, l7_policies, healthmonitor
+            # or loadbalancer in dict
             pool_dict = pool.to_dict(listeners=False, members=False,
-                                     healthmonitor=False, loadbalancer=False,
-                                     listener=False, loadbalancer_id=False)
+                                     l7_policies=False, healthmonitor=False,
+                                     loadbalancer=False, listener=False,
+                                     loadbalancer_id=False)
             pool_dict['healthmonitor_id'] = hm_db.id
             self.update_pool(context, pool_id, pool_dict)
             hm_db = self._get_resource(context, models.HealthMonitorV2,
@@ -739,7 +813,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
     def create_healthmonitor(self, context, healthmonitor):
         with context.session.begin(subtransactions=True):
             self._load_id(context, healthmonitor)
-            healthmonitor['provisioning_status'] = constants.PENDING_CREATE
+            healthmonitor['provisioning_status'] = n_const.PENDING_CREATE
             hm_db_entry = models.HealthMonitorV2(**healthmonitor)
             context.session.add(hm_db_entry)
         return data_models.HealthMonitor.from_sqlalchemy_model(hm_db_entry)
@@ -770,6 +844,16 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.HealthMonitor.from_sqlalchemy_model(hm_db)
                 for hm_db in hm_dbs]
 
+    def get_healthmonitors_as_api_dict(self, context, filters=None):
+        options = (
+            subqueryload(models.HealthMonitorV2.pool)
+        )
+        filters = filters or {}
+        hm_dbs = self._get_resources(context, models.HealthMonitorV2,
+                                     filters=filters, options=options)
+        return [hm_db.to_api_dict
+                for hm_db in hm_dbs]
+
     def update_loadbalancer_stats(self, context, loadbalancer_id, stats_data):
         stats_data = stats_data or {}
         with context.session.begin(subtransactions=True):
@@ -800,7 +884,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
 
             self._load_id(context, l7policy)
 
-            l7policy['provisioning_status'] = constants.PENDING_CREATE
+            l7policy['provisioning_status'] = n_const.PENDING_CREATE
 
             l7policy_db = models.L7Policy(**l7policy)
             # MySQL int fields are by default 32-bit whereas handy system
@@ -836,7 +920,11 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
                     context, models.Listener, listener_id)
                 l7policy_db = listener_db.l7_policies.pop(
                     l7policy_db.position - 1)
-
+                # NOTE(ihrachys) create a new policy model because the one just
+                # popped off the list became expired thanks to neutron db code
+                # that automatically expires relationships when foreign keys
+                # become obsolete
+                l7policy_db = models.L7Policy(**dict(l7policy_db))
                 l7policy_db.update(l7policy)
                 listener_db.l7_policies.insert(l7policy['position'] - 1,
                                                l7policy_db)
@@ -863,6 +951,15 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.L7Policy.from_sqlalchemy_model(l7policy_db)
                 for l7policy_db in l7policy_dbs]
 
+    def get_l7policies_as_api_dict(self, context, filters=None):
+        options = (
+            subqueryload(models.L7Policy.rules)
+        )
+        l7policy_dbs = self._get_resources(context, models.L7Policy,
+                                           filters=filters, options=options)
+        return [l7policy_db.to_api_dict
+                for l7policy_db in l7policy_dbs]
+
     def create_l7policy_rule(self, context, rule, l7policy_id):
         with context.session.begin(subtransactions=True):
             if not self._resource_exists(context, models.L7Policy,
@@ -872,7 +969,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             self._validate_l7rule_data(context, rule)
             self._load_id(context, rule)
             rule['l7policy_id'] = l7policy_id
-            rule['provisioning_status'] = constants.PENDING_CREATE
+            rule['provisioning_status'] = n_const.PENDING_CREATE
             rule_db = models.L7Rule(**rule)
             context.session.add(rule_db)
         return data_models.L7Rule.from_sqlalchemy_model(rule_db)
@@ -922,12 +1019,25 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.L7Rule.from_sqlalchemy_model(rule_db)
                 for rule_db in rule_dbs]
 
+    def get_l7policy_rules_as_api_dict(
+            self, context, l7policy_id, filters=None):
+        options = (
+            subqueryload(models.L7Rule.policy)
+        )
+        if filters:
+            filters.update(filters)
+        else:
+            filters = {'l7policy_id': [l7policy_id]}
+        rule_dbs = self._get_resources(context, models.L7Rule,
+                                       filters=filters, options=options)
+        return [rule_db.to_api_dict
+                for rule_db in rule_dbs]
+
 
 def _prevent_lbaasv2_port_delete_callback(resource, event, trigger, **kwargs):
     context = kwargs['context']
     port_id = kwargs['port_id']
     port_check = kwargs['port_check']
-    lbaasv2plugin = manager.NeutronManager.get_service_plugins().get(
-                         constants.LOADBALANCERV2)
+    lbaasv2plugin = directory.get_plugin(pg_const.LOADBALANCERV2)
     if lbaasv2plugin and port_check:
         lbaasv2plugin.db.prevent_lbaasv2_port_deletion(context, port_id)
