@@ -13,20 +13,19 @@
 #    under the License.
 
 import copy
-import netaddr
 import threading
 import time
 
-from neutron import context
-from neutron.plugins.common import constants
+import netaddr
 from neutron_lib import constants as n_constants
+from neutron_lib import context
+from neutron_lib.plugins import constants as pg_constants
 from oslo_config import cfg
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 from oslo_utils import excutils
 from six.moves import queue as Queue
 
-from neutron_lbaas._i18n import _LE, _LW, _LI
 import neutron_lbaas.common.cert_manager
 from neutron_lbaas.drivers.radware import base_v2_driver
 from neutron_lbaas.drivers.radware import exceptions as r_exc
@@ -79,7 +78,7 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
     # Assumptions:
     # 1) We have only one worflow that takes care of l2-l4 and service creation
     # 2) The workflow template exists on the vDirect server
-    # 3) The workflow expose one operaion named 'update' (plus ctor and dtor)
+    # 3) The workflow expose one operation named 'update' (plus ctor and dtor)
     # 4) The 'update' operation gets the loadbalancer object graph as input
     # 5) The object graph is enehanced by our code before it is sent to the
     #    workflow
@@ -132,10 +131,17 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
             server=vdirect_address,
             secondary_server=sec_server,
             user=rad.vdirect_user,
-            password=rad.vdirect_password)
+            password=rad.vdirect_password,
+            port=rad.port,
+            ssl=rad.ssl,
+            ssl_verify_context=rad.ssl_verify_context,
+            timeout=rad.timeout,
+            base_uri=rad.base_uri)
         self.workflow_params['provision_service'] = rad_debug.provision_service
         self.workflow_params['configure_l3'] = rad_debug.configure_l3
         self.workflow_params['configure_l4'] = rad_debug.configure_l4
+        self.configure_allowed_address_pairs =\
+            rad.configure_allowed_address_pairs
 
         self.queue = Queue.Queue()
         self.completion_handler = OperationCompletionHandler(self.queue,
@@ -147,7 +153,7 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
 
     def _start_completion_handling_thread(self):
         if not self.completion_handler_started:
-            LOG.info(_LI('Starting operation completion handling thread'))
+            LOG.info('Starting operation completion handling thread')
             self.completion_handler.start()
             self.completion_handler_started = True
 
@@ -213,7 +219,8 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
         _rest_wrapper(self.rest_client.call(
             'POST', tmpl_resource,
             {'parameters': dict(self.workflow_params,
-                                service_params=service)},
+                                service_params=service),
+            'tenants': [lb.tenant_id]},
             TEMPLATE_HEADER))
 
     @log_helpers.log_method_call
@@ -275,7 +282,8 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
         # Build objects graph
         objects_graph = self._build_objects_graph(ctx, lb, data_model,
                                                   proxy_port_address,
-                                                  proxy_subnet)
+                                                  proxy_subnet,
+                                                  delete)
         LOG.debug("Radware vDirect LB object graph is " + str(objects_graph))
 
         wf_name = self._get_wf_name(lb)
@@ -307,8 +315,7 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
                 LOG.debug('Proxy port for LB %s was deleted', lb.id)
             except Exception:
                 with excutils.save_and_reraise_exception():
-                    LOG.error(_LE('Proxy port deletion for LB %s '
-                                'failed'), lb.id)
+                    LOG.error('Proxy port deletion for LB %s failed', lb.id)
             manager.successful_completion(ctx, lb, delete=True)
         else:
             oper = OperationAttributes(
@@ -321,23 +328,31 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
             self.queue.put_nowait(oper)
 
     def _build_objects_graph(self, ctx, lb, data_model,
-                             proxy_port_address, proxy_subnet):
+                             proxy_port_address, proxy_subnet,
+                             deleted):
         """Iterate over the LB model starting from root lb entity
         and build its JSON representtaion for vDirect
         """
+        deleted_ids = []
+        if deleted:
+            deleted_ids.append(data_model.id)
+
         graph = {}
         for prop in LOADBALANCER_PROPERTIES:
             graph[prop] = getattr(lb, prop, PROPERTY_DEFAULTS.get(prop))
 
         graph['pip_address'] = proxy_port_address
+        graph['configure_allowed_address_pairs'] =\
+            self.configure_allowed_address_pairs
 
         graph['listeners'] = []
         listeners = [
             listener for listener in lb.listeners
-            if listener.provisioning_status != constants.PENDING_DELETE and
+            if listener.provisioning_status != n_constants.PENDING_DELETE and
             (listener.default_pool and
              listener.default_pool.provisioning_status !=
-             constants.PENDING_DELETE and
+             n_constants.PENDING_DELETE and
+             listener.default_pool.id not in deleted_ids and
              listener.default_pool.members)]
         for listener in listeners:
             listener_dict = {}
@@ -382,7 +397,8 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
             listener_dict['l7_policies'] = []
             policies = [
                 policy for policy in listener.l7_policies
-                if policy.provisioning_status != constants.PENDING_DELETE]
+                if policy.provisioning_status != n_constants.PENDING_DELETE and
+                policy.id not in deleted_ids]
             for policy in policies:
                 policy_dict = {}
                 for prop in L7_POLICY_PROPERTIES:
@@ -391,7 +407,9 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
                 policy_dict['rules'] = []
                 rules = [
                     rule for rule in policy.rules
-                    if rule.provisioning_status != constants.PENDING_DELETE]
+                    if rule.provisioning_status !=
+                    n_constants.PENDING_DELETE and
+                    rule.id not in deleted_ids]
                 for rule in rules:
                     rule_dict = {}
                     for prop in L7_RULE_PROPERTIES:
@@ -401,26 +419,24 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
                 if policy_dict['rules']:
                     listener_dict['l7_policies'].append(policy_dict)
 
-            if (listener.default_pool and
-                listener.default_pool.provisioning_status !=
-                    constants.PENDING_DELETE):
-                def_pool_dict = {'id': listener.default_pool.id}
+            def_pool_dict = {'id': listener.default_pool.id}
 
-                if listener.default_pool.session_persistence:
-                    sess_pers_dict = {}
-                    for prop in SESSION_PERSISTENCY_PROPERTIES:
-                        sess_pers_dict[prop] = getattr(
-                            listener.default_pool.session_persistence, prop,
-                            PROPERTY_DEFAULTS.get(prop))
-                    def_pool_dict['sessionpersistence'] = sess_pers_dict
-                listener_dict['default_pool'] = def_pool_dict
+            if listener.default_pool.session_persistence:
+                sess_pers_dict = {}
+                for prop in SESSION_PERSISTENCY_PROPERTIES:
+                    sess_pers_dict[prop] = getattr(
+                        listener.default_pool.session_persistence, prop,
+                        PROPERTY_DEFAULTS.get(prop))
+                def_pool_dict['sessionpersistence'] = sess_pers_dict
+            listener_dict['default_pool'] = def_pool_dict
 
             graph['listeners'].append(listener_dict)
 
         graph['pools'] = []
         pools = [
             pool for pool in lb.pools
-            if pool.provisioning_status != constants.PENDING_DELETE]
+            if pool.provisioning_status != n_constants.PENDING_DELETE and
+            pool.id not in deleted_ids]
         for pool in pools:
             pool_dict = {}
             for prop in POOL_PROPERTIES:
@@ -430,7 +446,8 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
 
             if (pool.healthmonitor and
                 pool.healthmonitor.provisioning_status !=
-                constants.PENDING_DELETE):
+                n_constants.PENDING_DELETE and
+                pool.healthmonitor.id not in deleted_ids):
                 hm_dict = {}
                 for prop in HEALTH_MONITOR_PROPERTIES:
                     hm_dict[prop] = getattr(
@@ -441,7 +458,8 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
             pool_dict['members'] = []
             members = [
                 member for member in pool.members
-                if member.provisioning_status != constants.PENDING_DELETE]
+                if member.provisioning_status != n_constants.PENDING_DELETE and
+                member.id not in deleted_ids]
             for member in members:
                 member_dict = {}
                 for prop in MEMBER_PROPERTIES:
@@ -480,8 +498,8 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
         """
         proxy_port = self._get_proxy_port(ctx, lb)
         if proxy_port:
-            LOG.info(_LI('LB %(lb_id)s proxy port exists on subnet \
-                     %(subnet_id)s with ip address %(ip_address)s') %
+            LOG.info('LB %(lb_id)s proxy port exists on subnet '
+                     '%(subnet_id)s with ip address %(ip_address)s' %
                      {'lb_id': lb.id, 'subnet_id': proxy_port['subnet_id'],
                       'ip_address': proxy_port['ip_address']})
             return proxy_port
@@ -496,15 +514,15 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
             'mac_address': n_constants.ATTR_NOT_SPECIFIED,
             'admin_state_up': False,
             'device_id': '',
-            'device_owner': 'neutron:' + constants.LOADBALANCERV2,
+            'device_owner': 'neutron:' + pg_constants.LOADBALANCERV2,
             'fixed_ips': [{'subnet_id': proxy_port_subnet_id}]
         }
         proxy_port = self.plugin.db._core_plugin.create_port(
             ctx, {'port': proxy_port_data})
         proxy_port_ip_data = proxy_port['fixed_ips'][0]
 
-        LOG.info(_LI('LB %(lb_id)s proxy port created on subnet %(subnet_id)s \
-                 with ip address %(ip_address)s') %
+        LOG.info('LB %(lb_id)s proxy port created on subnet %(subnet_id)s '
+                 'with ip address %(ip_address)s' %
                  {'lb_id': lb.id, 'subnet_id': proxy_port_ip_data['subnet_id'],
                   'ip_address': proxy_port_ip_data['ip_address']})
 
@@ -529,8 +547,8 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
             proxy_port = ports[0]
             proxy_port_ip_data = proxy_port['fixed_ips'][0]
             try:
-                LOG.info(_LI('Deleting LB %(lb_id)s proxy port on subnet  \
-                             %(subnet_id)s with ip address %(ip_address)s') %
+                LOG.info('Deleting LB %(lb_id)s proxy port on subnet  '
+                         '%(subnet_id)s with ip address %(ip_address)s' %
                          {'lb_id': lb.id,
                           'subnet_id': proxy_port_ip_data['subnet_id'],
                           'ip_address': proxy_port_ip_data['ip_address']})
@@ -540,7 +558,7 @@ class RadwareLBaaSV2Driver(base_v2_driver.RadwareLBaaSBaseV2Driver):
             except Exception as exception:
                 # stop exception propagation, nport may have
                 # been deleted by other means
-                LOG.warning(_LW('Proxy port deletion failed: %r'),
+                LOG.warning('Proxy port deletion failed: %r',
                             exception)
 
     def _accomplish_member_static_route_data(self,
@@ -610,13 +628,13 @@ class OperationCompletionHandler(threading.Thread):
                 else:
                     msg = "unknown"
                 error_params = {"operation": oper, "msg": msg}
-                LOG.error(_LE(
-                    'Operation %(operation)s failed. Reason: %(msg)s'),
+                LOG.error(
+                    'Operation %(operation)s failed. Reason: %(msg)s',
                     error_params)
-                oper.status = constants.ERROR
+                oper.status = n_constants.ERROR
                 OperationCompletionHandler._run_post_failure_function(oper)
             else:
-                oper.status = constants.ACTIVE
+                oper.status = n_constants.ACTIVE
                 OperationCompletionHandler._run_post_success_function(oper)
 
         return completed
@@ -652,8 +670,8 @@ class OperationCompletionHandler(threading.Thread):
             except Queue.Empty:
                 continue
             except Exception:
-                LOG.error(_LE(
-                    "Exception was thrown inside OperationCompletionHandler"))
+                LOG.error(
+                    "Exception was thrown inside OperationCompletionHandler")
 
     @staticmethod
     def _run_post_success_function(oper):
@@ -668,8 +686,8 @@ class OperationCompletionHandler(threading.Thread):
                       repr(oper))
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Post-operation success function failed '
-                             'for operation %s'),
+                LOG.error('Post-operation success function failed '
+                          'for operation %s',
                           repr(oper))
 
     @staticmethod
@@ -682,8 +700,8 @@ class OperationCompletionHandler(threading.Thread):
                       repr(oper))
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Post-operation failure function failed '
-                              'for operation %s'),
+                LOG.error('Post-operation failure function failed '
+                          'for operation %s',
                           repr(oper))
 
 
