@@ -16,21 +16,23 @@
 
 import abc
 
+from neutron_lib.api import converters
+from neutron_lib.api import extensions as api_extensions
+from neutron_lib.api import validators
+from neutron_lib import constants as n_constants
+from neutron_lib.db import constants as db_const
+from neutron_lib import exceptions as nexception
+from neutron_lib.plugins import constants
+from neutron_lib.plugins import directory
+from neutron_lib.services import base as service_base
 from oslo_config import cfg
 from oslo_log import log as logging
 import six
 
 from neutron.api import extensions
-from neutron.api.v2 import attributes as attr
 from neutron.api.v2 import base
 from neutron.api.v2 import resource_helper
-from neutron import manager
-from neutron.plugins.common import constants
-from neutron.services import service_base
-from neutron_lib.api import converters
-from neutron_lib.api import validators
-from neutron_lib import constants as n_constants
-from neutron_lib import exceptions as nexception
+from neutron.quota import resource_registry
 
 from neutron_lbaas._i18n import _
 from neutron_lbaas.services.loadbalancer import constants as lb_const
@@ -56,7 +58,7 @@ class DelayOrTimeoutInvalid(nexception.BadRequest):
 
 
 class EntityInUse(nexception.InUse):
-    message = _("%(entity_using)s %(id)s is using this %(entity_in_use)s")
+    message = _("%(entity_using)s is using %(entity_in_use)s %(id)s")
 
 
 class OnePoolPerListener(nexception.InUse):
@@ -144,6 +146,17 @@ def _validate_connection_limit(data, min_value=lb_const.MIN_CONNECT_VALUE):
 
 validators.validators['type:connection_limit'] = _validate_connection_limit
 
+
+def _validate_db_limit(data, max_value=db_const.DB_INTEGER_MAX_VALUE):
+    if int(data) > max_value:
+        msg = (_("'%(data)s' is not a valid value, "
+                 "because it is more than %(max_value)s") %
+               {'data': data, 'max_value': max_value})
+        LOG.debug(msg)
+        return msg
+
+validators.validators['type:db_out_of_bounds'] = _validate_db_limit
+
 RESOURCE_ATTRIBUTE_MAP = {
     'loadbalancers': {
         'id': {'allow_post': False, 'allow_put': False,
@@ -151,16 +164,17 @@ RESOURCE_ATTRIBUTE_MAP = {
                'is_visible': True,
                'primary_key': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'default': '',
                  'is_visible': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:not_empty_string':
-                                   attr.TENANT_ID_MAX_LEN},
+                                   db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
                       'is_visible': True},
         'description': {'allow_post': True, 'allow_put': True,
-                        'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
+                        'validate': {
+                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'vip_subnet_id': {'allow_post': True, 'allow_put': False,
                           'validate': {'type:uuid': None},
@@ -186,7 +200,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                              'is_visible': True},
         'flavor_id': {'allow_post': True, 'allow_put': False,
                       'is_visible': True,
-                      'validate': {'type:string': attr.NAME_MAX_LEN},
+                      'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                       'default': n_constants.ATTR_NOT_SPECIFIED}
     },
     'listeners': {
@@ -196,15 +210,16 @@ RESOURCE_ATTRIBUTE_MAP = {
                'primary_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:not_empty_string':
-                                   attr.TENANT_ID_MAX_LEN},
+                                   db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
                       'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'default': '',
                  'is_visible': True},
         'description': {'allow_post': True, 'allow_put': True,
-                        'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
+                        'validate': {
+                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'loadbalancer_id': {'allow_post': True, 'allow_put': False,
                             'validate': {'type:uuid': None},
@@ -249,14 +264,15 @@ RESOURCE_ATTRIBUTE_MAP = {
                'primary_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:not_empty_string':
-                                   attr.TENANT_ID_MAX_LEN},
+                                   db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
                       'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'is_visible': True, 'default': ''},
         'description': {'allow_post': True, 'allow_put': True,
-                        'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
+                        'validate': {
+                            'type:string': db_const.DESCRIPTION_FIELD_SIZE},
                         'is_visible': True, 'default': ''},
         'listener_id': {'allow_post': True, 'allow_put': False,
                         'validate': {'type:uuid': None},
@@ -300,7 +316,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                'primary_key': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:not_empty_string':
-                                   attr.TENANT_ID_MAX_LEN},
+                                   db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
                       'is_visible': True},
         'pool_id': {'allow_post': True, 'allow_put': False,
@@ -313,7 +329,10 @@ RESOURCE_ATTRIBUTE_MAP = {
                      'type:values': lb_const.SUPPORTED_HEALTH_MONITOR_TYPES},
                  'is_visible': True},
         'delay': {'allow_post': True, 'allow_put': True,
-                  'validate': {'type:non_negative': None},
+                  'validate': {
+                      'type:db_out_of_bounds':
+                          db_const.DB_INTEGER_MAX_VALUE,
+                      'type:non_negative': None},
                   'convert_to': converters.convert_to_int,
                   'is_visible': True},
         'timeout': {'allow_post': True, 'allow_put': True,
@@ -348,7 +367,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                            'convert_to': converters.convert_to_boolean,
                            'is_visible': True},
         'name': {'allow_post': True, 'allow_put': True,
-                 'validate': {'type:string': attr.NAME_MAX_LEN},
+                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                  'default': '',
                  'is_visible': True}
     }
@@ -365,7 +384,7 @@ SUB_RESOURCE_ATTRIBUTE_MAP = {
                    'primary_key': True},
             'tenant_id': {'allow_post': True, 'allow_put': False,
                           'validate': {'type:not_empty_string':
-                                       attr.TENANT_ID_MAX_LEN},
+                                       db_const.PROJECT_ID_FIELD_SIZE},
                           'required_by_policy': True,
                           'is_visible': True},
             'address': {'allow_post': True, 'allow_put': False,
@@ -388,7 +407,7 @@ SUB_RESOURCE_ATTRIBUTE_MAP = {
                           'validate': {'type:uuid': None},
                           'is_visible': True},
             'name': {'allow_post': True, 'allow_put': True,
-                     'validate': {'type:string': attr.NAME_MAX_LEN},
+                     'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                      'default': '',
                      'is_visible': True},
         }
@@ -399,29 +418,54 @@ SUB_RESOURCE_ATTRIBUTE_MAP = {
 lbaasv2_quota_opts = [
     cfg.IntOpt('quota_loadbalancer',
                default=10,
+               deprecated_for_removal=True,
+               deprecated_since='Queens',
+               deprecated_reason='The neutron-lbaas project is now '
+                                 'deprecated. See: https://wiki.openstack.org/'
+                                 'wiki/Neutron/LBaaS/Deprecation',
                help=_('Number of LoadBalancers allowed per tenant. '
                       'A negative value means unlimited.')),
     cfg.IntOpt('quota_listener',
                default=-1,
+               deprecated_for_removal=True,
+               deprecated_since='Queens',
+               deprecated_reason='The neutron-lbaas project is now '
+                                 'deprecated. See: https://wiki.openstack.org/'
+                                 'wiki/Neutron/LBaaS/Deprecation',
                help=_('Number of Loadbalancer Listeners allowed per tenant. '
                       'A negative value means unlimited.')),
     cfg.IntOpt('quota_pool',
                default=10,
+               deprecated_for_removal=True,
+               deprecated_since='Queens',
+               deprecated_reason='The neutron-lbaas project is now '
+                                 'deprecated. See: https://wiki.openstack.org/'
+                                 'wiki/Neutron/LBaaS/Deprecation',
                help=_('Number of pools allowed per tenant. '
                       'A negative value means unlimited.')),
     cfg.IntOpt('quota_member',
                default=-1,
+               deprecated_for_removal=True,
+               deprecated_since='Queens',
+               deprecated_reason='The neutron-lbaas project is now '
+                                 'deprecated. See: https://wiki.openstack.org/'
+                                 'wiki/Neutron/LBaaS/Deprecation',
                help=_('Number of pool members allowed per tenant. '
                       'A negative value means unlimited.')),
     cfg.IntOpt('quota_healthmonitor',
                default=-1,
+               deprecated_for_removal=True,
+               deprecated_since='Queens',
+               deprecated_reason='The neutron-lbaas project is now '
+                                 'deprecated. See: https://wiki.openstack.org/'
+                                 'wiki/Neutron/LBaaS/Deprecation',
                help=_('Number of health monitors allowed per tenant. '
                       'A negative value means unlimited.'))
 ]
 cfg.CONF.register_opts(lbaasv2_quota_opts, 'QUOTAS')
 
 
-class Loadbalancerv2(extensions.ExtensionDescriptor):
+class Loadbalancerv2(api_extensions.ExtensionDescriptor):
 
     @classmethod
     def get_name(cls):
@@ -433,15 +477,11 @@ class Loadbalancerv2(extensions.ExtensionDescriptor):
 
     @classmethod
     def get_description(cls):
-        return "Extension for LoadBalancing service v2"
-
-    @classmethod
-    def get_namespace(cls):
-        return "http://wiki.openstack.org/neutron/LBaaS/API_2.0"
+        return "Extension for LoadBalancing service v2 (deprecated)"
 
     @classmethod
     def get_updated(cls):
-        return "2014-06-18T10:00:00-00:00"
+        return "2018-01-31T10:00:00-00:00"
 
     @classmethod
     def get_resources(cls):
@@ -449,22 +489,20 @@ class Loadbalancerv2(extensions.ExtensionDescriptor):
             {}, RESOURCE_ATTRIBUTE_MAP)
         action_map = {'loadbalancer': {'stats': 'GET', 'statuses': 'GET'}}
         plural_mappings['members'] = 'member'
+        resource_registry.register_resource_by_name('member', 'members')
         plural_mappings['sni_container_refs'] = 'sni_container_ref'
         plural_mappings['sni_container_ids'] = 'sni_container_id'
-        attr.PLURALS.update(plural_mappings)
         resources = resource_helper.build_resource_info(
             plural_mappings,
             RESOURCE_ATTRIBUTE_MAP,
             constants.LOADBALANCERV2,
             action_map=action_map,
             register_quota=True)
-        plugin = manager.NeutronManager.get_service_plugins()[
-            constants.LOADBALANCERV2]
+        plugin = directory.get_plugin(constants.LOADBALANCERV2)
         for collection_name in SUB_RESOURCE_ATTRIBUTE_MAP:
             # Special handling needed for sub-resources with 'y' ending
             # (e.g. proxies -> proxy)
             resource_name = collection_name[:-1]
-
             parent = SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].get('parent')
             params = SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].get(
                 'parameters')
@@ -503,14 +541,11 @@ class Loadbalancerv2(extensions.ExtensionDescriptor):
 @six.add_metaclass(abc.ABCMeta)
 class LoadBalancerPluginBaseV2(service_base.ServicePluginBase):
 
-    def get_plugin_name(self):
-        return constants.LOADBALANCERV2
-
     def get_plugin_type(self):
         return constants.LOADBALANCERV2
 
     def get_plugin_description(self):
-        return 'LoadBalancer service plugin v2'
+        return 'LoadBalancer service plugin v2 (deprecated)'
 
     @abc.abstractmethod
     def get_loadbalancers(self, context, filters=None, fields=None):
@@ -631,6 +666,7 @@ class LoadBalancerPluginBaseV2(service_base.ServicePluginBase):
     def statuses(self, context, loadbalancer_id):
         pass
 
+    @abc.abstractmethod
     def get_l7policies(self, context, filters=None, fields=None):
         pass
 
