@@ -15,13 +15,14 @@
 
 import time
 
+from neutron_lib import constants as n_constants
 from oslo_log import log as logging
 from tempest.api.network import base
 from tempest import config
 from tempest.lib.common.utils import test_utils
 from tempest.lib import exceptions
 
-from neutron_lbaas._i18n import _, _LI
+from neutron_lbaas._i18n import _
 from neutron_lbaas.tests.tempest.v2.clients import health_monitors_client
 from neutron_lbaas.tests.tempest.v2.clients import listeners_client
 from neutron_lbaas.tests.tempest.v2.clients import load_balancers_client
@@ -65,6 +66,10 @@ class BaseTestCase(base.BaseNetworkTest):
     def resource_setup(cls):
         super(BaseTestCase, cls).resource_setup()
 
+        cls.listener_protocol = CONF.lbaas.default_listener_protocol
+        cls.pool_protocol = CONF.lbaas.default_pool_protocol
+        cls.hm_protocol = CONF.lbaas.default_health_monitor_protocol
+
         mgr = cls.get_client_manager()
         auth_provider = mgr.auth_provider
         client_args = _setup_client_args(auth_provider)
@@ -120,6 +125,29 @@ class BaseTestCase(base.BaseNetworkTest):
             test_utils.call_and_ignore_notfound_exc(
                 cls._delete_load_balancer, lb_id)
 
+        # Wait for all LBs to be fully deleted
+        for lb_id in cls._lbs_to_delete:
+            cls._wait_for_load_balancer_status(
+                lb_id, provisioning_status='DELETED', operating_status=None,
+                delete=True)
+
+        # Wait for straggling port deletes to finish (neutron will sometimes
+        # claim they are deleted before they are totally finished deleting)
+        remaining_ports = cls.ports_client.list_ports(
+            network_id=cls.network['id']).get('ports')
+        tries = 1
+        while remaining_ports and tries <= 10:
+            LOG.warning("Ports still exist on network %s (try #%s): %s",
+                        cls.network['id'], tries, remaining_ports)
+            time.sleep(10)
+            remaining_ports = cls.ports_client.list_ports(
+                network_id=cls.network['id']).get('ports')
+            tries += 1
+
+        if remaining_ports:
+            LOG.error("Failed to remove all remaining ports on network %s. "
+                      "Subnet deletion will probably fail.", cls.network['id'])
+
         super(BaseTestCase, cls).resource_cleanup()
 
     @classmethod
@@ -128,12 +156,12 @@ class BaseTestCase(base.BaseNetworkTest):
         super(BaseTestCase, cls).setUpClass()
 
     def setUp(cls):
-        cls.LOG.info(_LI('Starting: {0}').format(cls._testMethodName))
+        cls.LOG.info('Starting: {0}'.format(cls._testMethodName))
         super(BaseTestCase, cls).setUp()
 
     def tearDown(cls):
         super(BaseTestCase, cls).tearDown()
-        cls.LOG.info(_LI('Finished: {0}\n').format(cls._testMethodName))
+        cls.LOG.info('Finished: {0}\n'.format(cls._testMethodName))
 
     @classmethod
     def _create_load_balancer(cls, wait=True, **lb_kwargs):
@@ -142,8 +170,6 @@ class BaseTestCase(base.BaseNetworkTest):
             cls._wait_for_load_balancer_status(lb.get('id'))
 
         cls._lbs_to_delete.append(lb.get('id'))
-        port = cls.ports_client.show_port(lb['vip_port_id'])
-        cls.ports.append(port['port'])
         return lb
 
     @classmethod
@@ -177,6 +203,10 @@ class BaseTestCase(base.BaseNetworkTest):
         timeout = 600
         end_time = time.time() + timeout
         lb = {}
+        # When running with no-op drivers there is no actual health to
+        # observe, so disable operating_status checks when running no-op.
+        if CONF.lbaas.test_with_noop:
+            operating_status = None
         while time.time() < end_time:
             try:
                 lb = cls.load_balancers_client.get_load_balancer(
@@ -190,9 +220,11 @@ class BaseTestCase(base.BaseNetworkTest):
                             _("loadbalancer {lb_id} not"
                               " found").format(
                                   lb_id=load_balancer_id))
-                if (lb.get('provisioning_status') == provisioning_status and
-                        lb.get('operating_status') == operating_status):
-                    break
+                if lb.get('provisioning_status') == provisioning_status:
+                    if operating_status is None:
+                        break
+                    elif lb.get('operating_status') == operating_status:
+                        break
                 time.sleep(interval_time)
             except exceptions.NotFound:
                 # if wait is for delete operation do break
@@ -221,6 +253,27 @@ class BaseTestCase(base.BaseNetworkTest):
                           operating_status=operating_status))
         return lb
 
+    @classmethod
+    def _wait_for_neutron_port_delete(cls, port_id):
+        """
+        Neutron port deletion is asynchronous. This method waits, up to a
+        timeout for the port deletion to complete.
+        """
+        interval_time = 1
+        timeout = 600
+        end_time = time.time() + timeout
+        while time.time() < end_time:
+            try:
+                cls.ports_client.show_port(port_id)
+                time.sleep(interval_time)
+            except exceptions.NotFound:
+                break
+        else:
+            raise exceptions.TimeoutException(
+                    _("Waited for port {port_id} to be deleted for "
+                      "{timeout} seconds but can still observe that it "
+                      "exists.").format(port_id=port_id, timeout=timeout))
+
     @classmethod
     def _create_listener(cls, wait=True, **listener_kwargs):
         listener = cls.listeners_client.create_listener(**listener_kwargs)
@@ -318,7 +371,12 @@ class BaseTestCase(base.BaseNetworkTest):
         statuses = cls.load_balancers_client.get_load_balancer_status_tree(
             load_balancer_id=load_balancer_id)
         load_balancer = statuses['loadbalancer']
-        assert 'ONLINE' == load_balancer['operating_status']
+
+        # When running with no-op drivers there is no actual health to
+        # observe, so disable operating_status checks when running no-op.
+        if not CONF.lbaas.test_with_noop:
+            assert 'ONLINE' == load_balancer['operating_status']
+
         assert 'ACTIVE' == load_balancer['provisioning_status']
 
         if listener_ids:
@@ -340,13 +398,16 @@ class BaseTestCase(base.BaseNetworkTest):
     @classmethod
     def _check_status_tree_thing(cls, actual_thing_ids, status_tree_things):
         found_things = 0
-        status_tree_things = status_tree_things
         assert len(actual_thing_ids) == len(status_tree_things)
         for actual_thing_id in actual_thing_ids:
             for status_tree_thing in status_tree_things:
                 if status_tree_thing['id'] == actual_thing_id:
-                    assert 'ONLINE' == (
-                        status_tree_thing['operating_status'])
+                    # When running with no-op drivers there is no actual
+                    # health to observe, so disable operating_status checks
+                    # when running no-op.
+                    if not CONF.lbaas.test_with_noop:
+                        assert 'ONLINE' == (
+                            status_tree_thing['operating_status'])
                     assert 'ACTIVE' == (
                         status_tree_thing['provisioning_status'])
                     found_things += 1
@@ -360,6 +421,17 @@ class BaseTestCase(base.BaseNetworkTest):
         )
         return name
 
+    def _test_provisioning_status_if_exists(self, created_obj, get_obj):
+        prov_status_old = created_obj.pop('provisioning_status', None)
+        prov_status_new = get_obj.pop('provisioning_status', None)
+
+        if prov_status_old:
+            self.assertIn('PENDING_', prov_status_old)
+        if prov_status_new:
+            self.assertEqual(n_constants.ACTIVE, prov_status_new)
+        if not created_obj.get('updated_at') and get_obj.get('updated_at'):
+            get_obj['updated_at'] = None
+
 
 class BaseAdminTestCase(BaseTestCase):
 
