@@ -13,9 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from tempest.common import utils
 from tempest.lib.common.utils import data_utils
+from tempest.lib import decorators
 from tempest.lib import exceptions as ex
-from tempest import test
 
 from neutron_lbaas.tests.tempest.v2.api import base
 
@@ -36,7 +37,7 @@ class ListenersTestJSON(base.BaseTestCase):
     @classmethod
     def resource_setup(cls):
         super(ListenersTestJSON, cls).resource_setup()
-        if not test.is_extension_enabled('lbaas', 'network'):
+        if not utils.is_extension_enabled('lbaasv2', 'network'):
             msg = "lbaas extension not enabled."
             raise cls.skipException(msg)
         network_name = data_utils.rand_name('network-')
@@ -46,34 +47,40 @@ class ListenersTestJSON(base.BaseTestCase):
                                 'vip_subnet_id': cls.subnet['id']}
         cls.load_balancer = cls._create_active_load_balancer(
             **cls.create_lb_kwargs)
-        cls.protocol = 'HTTP'
         cls.port = 80
         cls.load_balancer_id = cls.load_balancer['id']
         cls.create_listener_kwargs = {'loadbalancer_id': cls.load_balancer_id,
-                                      'protocol': cls.protocol,
+                                      'protocol': cls.listener_protocol,
                                       'protocol_port': cls.port}
         cls.listener = cls._create_listener(**cls.create_listener_kwargs)
         cls.listener_id = cls.listener['id']
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_get_listener(self):
         """Test get listener"""
         listener = self.listeners_client.get_listener(
             self.listener_id)
+        self._test_provisioning_status_if_exists(self.listener, listener)
         self.assertEqual(self.listener, listener)
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='smoke')
+    def _prep_list_comparison(self, single, obj_list):
+        for obj in obj_list:
+            if not single.get('updated_at') and obj.get('updated_at'):
+                obj['updated_at'] = None
+            self._test_provisioning_status_if_exists(single, obj)
+
     def test_list_listeners(self):
         """Test get listeners with one listener"""
         listeners = self.listeners_client.list_listeners()
         self.assertEqual(len(listeners), 1)
+        self._prep_list_comparison(self.listener, listeners)
         self.assertIn(self.listener, listeners)
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_list_listeners_two(self):
         """Test get listeners with two listeners"""
         create_new_listener_kwargs = self.create_listener_kwargs
@@ -87,11 +94,13 @@ class ListenersTestJSON(base.BaseTestCase):
             listener_ids=[self.listener_id, new_listener_id])
         listeners = self.listeners_client.list_listeners()
         self.assertEqual(len(listeners), 2)
+        self._prep_list_comparison(self.listener, listeners)
+        self._prep_list_comparison(new_listener, listeners)
         self.assertIn(self.listener, listeners)
         self.assertIn(new_listener, listeners)
         self.assertNotEqual(self.listener, new_listener)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_create_listener(self):
         """Test create listener"""
         create_new_listener_kwargs = self.create_listener_kwargs
@@ -105,20 +114,22 @@ class ListenersTestJSON(base.BaseTestCase):
             listener_ids=[self.listener_id, new_listener_id])
         listener = self.listeners_client.get_listener(
             new_listener_id)
+        self._test_provisioning_status_if_exists(new_listener, listener)
         self.assertEqual(new_listener, listener)
+        self._test_provisioning_status_if_exists(self.listener, new_listener)
         self.assertNotEqual(self.listener, new_listener)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_missing_field_loadbalancer(self):
         """Test create listener with a missing required field loadbalancer"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           protocol_port=self.port,
-                          protocol=self.protocol)
+                          protocol=self.listener_protocol)
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_missing_field_protocol(self):
         """Test create listener with a missing required field protocol"""
         self.assertRaises(ex.BadRequest,
@@ -128,17 +139,16 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_missing_field_protocol_port(self):
         """Test create listener with a missing required field protocol_port"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           loadbalancer_id=self.load_balancer_id,
-                          protocol=self.protocol)
+                          protocol=self.listener_protocol)
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='smoke')
     def test_create_listener_missing_admin_state_up(self):
         """Test create listener with a missing admin_state_up field"""
         create_new_listener_kwargs = self.create_listener_kwargs
@@ -152,21 +162,22 @@ class ListenersTestJSON(base.BaseTestCase):
             listener_ids=[self.listener_id, new_listener_id])
         listener = self.listeners_client.get_listener(
             new_listener_id)
+        self._test_provisioning_status_if_exists(new_listener, listener)
         self.assertEqual(new_listener, listener)
         self.assertTrue(new_listener['admin_state_up'])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_invalid_load_balancer_id(self):
         """Test create listener with an invalid load_balancer_id"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           loadbalancer_id="234*",
                           protocol_port=self.port,
-                          protocol=self.protocol)
+                          protocol=self.listener_protocol)
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_invalid_protocol(self):
         """Test create listener with an invalid protocol"""
         self.assertRaises(ex.BadRequest,
@@ -177,65 +188,66 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_invalid_protocol_port(self):
         """Test create listener with an invalid protocol_port"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           loadbalancer_id=self.load_balancer_id,
                           protocol_port="9999999",
-                          protocol=self.protocol)
+                          protocol=self.listener_protocol)
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_invalid_admin_state_up(self):
         """Test update listener with an invalid admin_state_up"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           protocol_port=self.port,
-                          protocol=self.protocol,
+                          protocol=self.listener_protocol,
                           admin_state_up="abc123")
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.skip_because(bug="1468457")
+    @decorators.attr(type='negative')
     def test_create_listener_invalid_tenant_id(self):
         """Test create listener with an invalid tenant id"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           loadbalancer_id=self.load_balancer_id,
                           protocol_port=self.port,
-                          protocol=self.protocol,
+                          protocol=self.listener_protocol,
                           tenant_id="&^%123")
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_invalid_name(self):
         """Test create listener with an invalid name"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           loadbalancer_id=self.load_balancer_id,
                           protocol_port=self.port,
-                          protocol=self.protocol,
+                          protocol=self.listener_protocol,
                           name='a' * 256)
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_invalid_description(self):
         """Test create listener with an invalid description"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           loadbalancer_id=self.load_balancer_id,
                           protocol_port=self.port,
-                          protocol=self.protocol,
+                          protocol=self.listener_protocol,
                           description='a' * 256)
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_invalid_connection_limit(self):
         """Test create listener with an invalid value for connection
         _limit field
@@ -244,23 +256,23 @@ class ListenersTestJSON(base.BaseTestCase):
                           self._create_listener,
                           loadbalancer_id=self.load_balancer_id,
                           protocol_port=self.port,
-                          protocol=self.protocol,
+                          protocol=self.listener_protocol,
                           connection_limit="&^%123")
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_empty_load_balancer_id(self):
         """Test create listener with an empty load_balancer_id"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           loadbalancer_id="",
                           protocol_port=self.port,
-                          protocol=self.protocol)
+                          protocol=self.listener_protocol)
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_empty_protocol(self):
         """Test create listener with an empty protocol"""
         self.assertRaises(ex.BadRequest,
@@ -271,41 +283,41 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_empty_protocol_port(self):
         """Test create listener with an empty protocol_port"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           loadbalancer_id=self.load_balancer_id,
                           protocol_port="",
-                          protocol=self.protocol)
+                          protocol=self.listener_protocol)
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_empty_admin_state_up(self):
         """Test update listener with an empty  admin_state_up"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           protocol_port=self.port,
-                          protocol=self.protocol,
+                          protocol=self.listener_protocol,
                           admin_state_up="")
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.skip_because(bug="1468457")
+    @decorators.attr(type='negative')
     def test_create_listener_empty_tenant_id(self):
         """Test create listener with an empty tenant id"""
         self.assertRaises(ex.BadRequest,
                           self._create_listener,
                           loadbalancer_id=self.load_balancer_id,
                           protocol_port=self.port,
-                          protocol=self.protocol,
+                          protocol=self.listener_protocol,
                           tenant_id="")
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='smoke')
     def test_create_listener_empty_name(self):
         """Test create listener with an empty name"""
         create_new_listener_kwargs = self.create_listener_kwargs
@@ -320,9 +332,9 @@ class ListenersTestJSON(base.BaseTestCase):
             listener_ids=[self.listener_id, new_listener_id])
         listener = self.listeners_client.get_listener(
             new_listener_id)
+        self._test_provisioning_status_if_exists(new_listener, listener)
         self.assertEqual(new_listener, listener)
 
-    @test.attr(type='smoke')
     def test_create_listener_empty_description(self):
         """Test create listener with an empty description"""
         create_new_listener_kwargs = self.create_listener_kwargs
@@ -337,9 +349,10 @@ class ListenersTestJSON(base.BaseTestCase):
             listener_ids=[self.listener_id, new_listener_id])
         listener = self.listeners_client.get_listener(
             new_listener_id)
+        self._test_provisioning_status_if_exists(new_listener, listener)
         self.assertEqual(new_listener, listener)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_empty_connection_limit(self):
         """Test create listener with an empty connection
         _limit field
@@ -348,12 +361,12 @@ class ListenersTestJSON(base.BaseTestCase):
                           self._create_listener,
                           loadbalancer_id=self.load_balancer_id,
                           protocol_port=self.port,
-                          protocol=self.protocol,
+                          protocol=self.listener_protocol,
                           connection_limit="")
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_listener_incorrect_attribute(self):
         """Test create a listener with an extra, incorrect field"""
         self.assertRaises(ex.BadRequest,
@@ -363,7 +376,7 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_update_listener(self):
         """Test update listener"""
         self._update_listener(self.listener_id,
@@ -374,7 +387,8 @@ class ListenersTestJSON(base.BaseTestCase):
             self.listener_id)
         self.assertEqual(listener.get('name'), 'new_name')
 
-    @test.attr(type='negative')
+    @decorators.skip_because(bug="1468457")
+    @decorators.attr(type='negative')
     def test_update_listener_invalid_tenant_id(self):
         """Test update listener with an invalid tenant id"""
         self.assertRaises(ex.BadRequest,
@@ -384,7 +398,7 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_listener_invalid_admin_state_up(self):
         """Test update a listener with an invalid admin_state_up"""
         self.assertRaises(ex.BadRequest,
@@ -394,7 +408,7 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_listener_invalid_name(self):
         """Test update a listener with an invalid name"""
         self.assertRaises(ex.BadRequest,
@@ -404,7 +418,7 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_listener_invalid_description(self):
         """Test update a listener with an invalid description"""
         self.assertRaises(ex.BadRequest,
@@ -414,7 +428,7 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_listener_invalid_connection_limit(self):
         """Test update a listener with an invalid connection_limit"""
         self.assertRaises(ex.BadRequest,
@@ -424,7 +438,7 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_listener_incorrect_attribute(self):
         """Test update a listener with an extra, incorrect field"""
         self.assertRaises(ex.BadRequest,
@@ -438,7 +452,6 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='smoke')
     def test_update_listener_missing_name(self):
         """Test update listener with a missing name"""
         old_listener = self.listeners_client.get_listener(
@@ -452,7 +465,6 @@ class ListenersTestJSON(base.BaseTestCase):
             self.listener_id)
         self.assertEqual(listener.get('name'), old_name)
 
-    @test.attr(type='smoke')
     def test_update_listener_missing_description(self):
         """Test update listener with a missing description"""
         old_listener = self.listeners_client.get_listener(
@@ -466,7 +478,6 @@ class ListenersTestJSON(base.BaseTestCase):
             self.listener_id)
         self.assertEqual(listener.get('description'), old_description)
 
-    @test.attr(type='smoke')
     def test_update_listener_missing_admin_state_up(self):
         """Test update listener with a missing admin_state_up"""
         old_listener = self.listeners_client.get_listener(
@@ -480,7 +491,6 @@ class ListenersTestJSON(base.BaseTestCase):
             self.listener_id)
         self.assertEqual(listener.get('admin_state_up'), old_admin_state_up)
 
-    @test.attr(type='smoke')
     def test_update_listener_missing_connection_limit(self):
         """Test update listener with a missing connection_limit"""
         old_listener = self.listeners_client.get_listener(
@@ -495,7 +505,8 @@ class ListenersTestJSON(base.BaseTestCase):
         self.assertEqual(listener.get('connection_limit'),
                          old_connection_limit)
 
-    @test.attr(type='negative')
+    @decorators.skip_because(bug="1468457")
+    @decorators.attr(type='negative')
     def test_update_listener_empty_tenant_id(self):
         """Test update listener with an empty tenant id"""
         self.assertRaises(ex.BadRequest,
@@ -505,7 +516,7 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_listener_empty_admin_state_up(self):
         """Test update a listener with an empty admin_state_up"""
         self.assertRaises(ex.BadRequest,
@@ -515,7 +526,6 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='smoke')
     def test_update_listener_empty_name(self):
         """Test update a listener with an empty name"""
         self._update_listener(self.listener_id,
@@ -526,7 +536,6 @@ class ListenersTestJSON(base.BaseTestCase):
             self.listener_id)
         self.assertEqual(listener.get('name'), "")
 
-    @test.attr(type='smoke')
     def test_update_listener_empty_description(self):
         """Test update a listener with an empty description"""
         self._update_listener(self.listener_id,
@@ -537,7 +546,7 @@ class ListenersTestJSON(base.BaseTestCase):
             self.listener_id)
         self.assertEqual(listener.get('description'), "")
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_listener_empty_connection_limit(self):
         """Test update a listener with an empty connection_limit"""
         self.assertRaises(ex.BadRequest,
@@ -547,7 +556,7 @@ class ListenersTestJSON(base.BaseTestCase):
         self._check_status_tree(load_balancer_id=self.load_balancer_id,
                                 listener_ids=[self.listener_id])
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_delete_listener(self):
         """Test delete listener"""
         create_new_listener_kwargs = self.create_listener_kwargs
@@ -559,7 +568,9 @@ class ListenersTestJSON(base.BaseTestCase):
             listener_ids=[self.listener_id, new_listener_id])
         listener = self.listeners_client.get_listener(
             new_listener_id)
+        self._test_provisioning_status_if_exists(new_listener, listener)
         self.assertEqual(new_listener, listener)
+        self._test_provisioning_status_if_exists(self.listener, new_listener)
         self.assertNotEqual(self.listener, new_listener)
         self._delete_listener(new_listener_id)
         self.assertRaises(ex.NotFound,
