@@ -13,10 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from tempest.common import utils
 from tempest import config
 from tempest.lib.common.utils import data_utils
+from tempest.lib import decorators
 from tempest.lib import exceptions as ex
-from tempest import test
 
 from neutron_lbaas.tests.tempest.v2.api import base
 
@@ -38,7 +39,7 @@ class MemberTestJSON(base.BaseTestCase):
     @classmethod
     def resource_setup(cls):
         super(MemberTestJSON, cls).resource_setup()
-        if not test.is_extension_enabled("lbaas", "network"):
+        if not utils.is_extension_enabled("lbaasv2", "network"):
             msg = "lbaas extension not enabled."
             raise cls.skipException(msg)
         network_name = data_utils.rand_name('network-')
@@ -52,9 +53,9 @@ class MemberTestJSON(base.BaseTestCase):
         cls.load_balancer_id = cls.load_balancer.get("id")
         cls.listener = cls._create_listener(
             loadbalancer_id=cls.load_balancer.get('id'),
-            protocol='HTTP', protocol_port=80)
+            protocol=cls.listener_protocol, protocol_port=80)
         cls.listener_id = cls.listener.get('id')
-        cls.pool = cls._create_pool(protocol='HTTP',
+        cls.pool = cls._create_pool(protocol=cls.pool_protocol,
                                     tenant_id=cls.tenant_id,
                                     lb_algorithm='ROUND_ROBIN',
                                     listener_id=cls.listener_id)
@@ -64,14 +65,13 @@ class MemberTestJSON(base.BaseTestCase):
     def resource_cleanup(cls):
         super(MemberTestJSON, cls).resource_cleanup()
 
-    @test.attr(type='smoke')
     def test_list_empty_members(self):
         """Test that pool members are empty."""
         members = self.members_client.list_members(self.pool_id)
         self.assertEmpty(members,
                          msg='Initial pool was supposed to be empty')
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_list_3_members(self):
         """Test that we can list members. """
         member_ips_exp = set([u"127.0.0.0", u"127.0.0.1", u"127.0.0.2"])
@@ -89,7 +89,7 @@ class MemberTestJSON(base.BaseTestCase):
         found_member_ips = set([m["address"] for m in members])
         self.assertEqual(found_member_ips, member_ips_exp)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_add_member(self):
         """Test that we can add a single member."""
         expect_empty_members = self.members_client.list_members(self.pool_id)
@@ -106,7 +106,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertTrue(member["admin_state_up"])
         self.assertEqual(1, member["weight"])
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_get_member(self):
         """Test that we can fetch a member by id."""
         member_opts = self.build_member_opts()
@@ -120,7 +120,6 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertEqual(member_opts["protocol_port"], member["protocol_port"])
         self.assertEqual(member_opts["subnet_id"], member["subnet_id"])
 
-    @test.attr(type='smoke')
     def test_create_member_missing_required_field_tenant_id(self):
         """Test if a non_admin user can create a member with tenant_id
         missing
@@ -132,7 +131,7 @@ class MemberTestJSON(base.BaseTestCase):
         member = self._create_member(self.pool_id, **member_opts)
         self.addCleanup(self._delete_member, self.pool_id, member['id'])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_missing_required_field_address(self):
         """Test create a member with missing field address"""
         member_opts = {}
@@ -141,7 +140,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_missing_required_field_protocol_port(self):
         """Test create a member with missing field protocol_port"""
         member_opts = {}
@@ -150,7 +149,8 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.skip_because(bug="1468457")
+    @decorators.attr(type='negative')
     def test_create_member_missing_required_field_subnet_id(self):
         """Test create a member with missing field subnet_id """
         member_opts = {}
@@ -159,14 +159,15 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_raises_BadRequest_when_missing_attrs_during_member_create(self):
         """Test failure on missing attributes on member create."""
         member_opts = {}
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.skip_because(bug="1468457")
+    @decorators.attr(type='negative')
     def test_create_member_invalid_tenant_id(self):
         """Test create member with invalid tenant_id"""
         member_opts = {}
@@ -177,7 +178,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_invalid_address(self):
         """Test create member with invalid address"""
         member_opts = {}
@@ -187,7 +188,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_invalid_protocol_port(self):
         """Test create member with invalid protocol_port"""
         member_opts = {}
@@ -197,7 +198,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_invalid_subnet_id(self):
         """Test create member with invalid subnet_id"""
         member_opts = {}
@@ -207,7 +208,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_invalid_admin_state_up(self):
         """Test create member with invalid admin_state_up"""
         member_opts = {}
@@ -218,7 +219,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_invalid_weight(self):
         """Test create member with invalid weight"""
         member_opts = {}
@@ -229,7 +230,8 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.skip_because(bug="1468457")  # Octavia does a floor()
+    @decorators.attr(type='negative')
     def test_create_member_nonint_weight(self):
         """Test create member with nonint weight"""
         member_opts = {}
@@ -240,7 +242,8 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.skip_because(bug="1468457")
+    @decorators.attr(type='negative')
     def test_create_member_empty_tenant_id(self):
         """Test create member with an empty tenant_id"""
         member_opts = {}
@@ -251,7 +254,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_empty_address(self):
         """Test create member with an empty address"""
         member_opts = {}
@@ -261,7 +264,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_empty_protocol_port(self):
         """Test create member with an empty protocol_port"""
         member_opts = {}
@@ -271,7 +274,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_empty_subnet_id(self):
         """Test create member with empty subnet_id"""
         member_opts = {}
@@ -281,7 +284,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_empty_admin_state_up(self):
         """Test create member with an empty admin_state_up"""
         member_opts = {}
@@ -292,7 +295,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_create_member_empty_weight(self):
         """Test create member with an empty weight"""
         member_opts = {}
@@ -303,7 +306,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_delete_member(self):
         """Test that we can delete a member by id."""
         member_opts = self.build_member_opts()
@@ -315,7 +318,7 @@ class MemberTestJSON(base.BaseTestCase):
         members = self.members_client.list_members(self.pool_id)
         self.assertEmpty(members)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_update_member(self):
         """Test that we can update a member."""
         member_opts = self.build_member_opts()
@@ -328,29 +331,27 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertEqual(1, member["weight"])
         # Lets overwrite the defaults
         member_opts = {"weight": 10, "admin_state_up": False}
-        member = self._update_member(self.pool_id, member_id,
-                                     **member_opts)
+        self._update_member(self.pool_id, member_id,
+                            **member_opts)
+        member = self.members_client.get_member(self.pool_id, member_id)
         # And make sure they stick
         self.assertFalse(member["admin_state_up"])
         self.assertEqual(10, member["weight"])
 
-    @test.attr(type='smoke')
     def test_update_member_missing_admin_state_up(self):
         """Test that we can update a member with missing admin_state_up."""
         member_opts = self.build_member_opts()
-        member = self._create_member(self.pool_id,
-                                     **member_opts)
+        member = self._create_member(self.pool_id, **member_opts)
         member_id = member["id"]
         self.addCleanup(self._delete_member, self.pool_id, member_id)
         self.assertTrue(member["admin_state_up"])
         self.assertEqual(1, member["weight"])
         member_opts = {"weight": 10}
-        member = self._update_member(self.pool_id, member_id,
-                                     **member_opts)
+        self._update_member(self.pool_id, member_id, **member_opts)
+        member = self.members_client.get_member(self.pool_id, member_id)
         self.assertTrue(member["admin_state_up"])
         self.assertEqual(10, member["weight"])
 
-    @test.attr(type='smoke')
     def test_update_member_missing_weight(self):
         """Test that we can update a member with missing weight."""
         member_opts = self.build_member_opts()
@@ -361,12 +362,12 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertTrue(member["admin_state_up"])
         self.assertEqual(1, member["weight"])
         member_opts = {"admin_state_up": False}
-        member = self._update_member(self.pool_id, member_id,
-                                     **member_opts)
+        self._update_member(self.pool_id, member_id, **member_opts)
+        member = self.members_client.get_member(self.pool_id, member_id)
         self.assertFalse(member["admin_state_up"])
         self.assertEqual(1, member["weight"])
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_member_invalid_admin_state_up(self):
         """Test that we can update a member with empty admin_state_up."""
         member_opts = self.build_member_opts()
@@ -380,7 +381,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._update_member,
                           self.pool_id, member_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_member_invalid_weight(self):
         """Test that we can update a member with an empty weight."""
         member_opts = self.build_member_opts()
@@ -394,7 +395,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._update_member,
                           self.pool_id, member_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_member_empty_admin_state_up(self):
         """Test that we can update a member with empty admin_state_up."""
         member_opts = self.build_member_opts()
@@ -408,7 +409,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._update_member,
                           self.pool_id, member_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_update_member_empty_weight(self):
         """Test that we can update a member with an empty weight."""
         member_opts = self.build_member_opts()
@@ -422,7 +423,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._update_member,
                           self.pool_id, member_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_raises_immutable_when_updating_immutable_attrs_on_member(self):
         """Test failure on immutable attribute on member create."""
         member_opts = self.build_member_opts()
@@ -435,7 +436,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._update_member,
                           self.pool_id, member_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_raises_exception_on_invalid_attr_on_create(self):
         """Test failure on invalid attribute on member create."""
         member_opts = self.build_member_opts()
@@ -443,7 +444,7 @@ class MemberTestJSON(base.BaseTestCase):
         self.assertRaises(ex.BadRequest, self._create_member,
                           self.pool_id, **member_opts)
 
-    @test.attr(type='negative')
+    @decorators.attr(type='negative')
     def test_raises_exception_on_invalid_attr_on_update(self):
         """Test failure on invalid attribute on member update."""
         member_opts = self.build_member_opts()
